@@ -21,6 +21,7 @@ from tempest.lib.services.image.v2 import images_client
 from tempest.lib.services.network import networks_client
 from tempest.lib.services.network import ports_client
 from tempest.lib.services.network import security_groups_client
+from tempest.lib.services.network import subnetpools_client
 from tempest.lib.services.network import subnets_client
 from tempest.lib.services.volume.v3 import volumes_client
 from tempest import manager
@@ -82,6 +83,8 @@ class Manager(manager.Manager):
             self.auth_provider, 'network', CONF.identity.region)
         self.subnets_client = subnets_client.SubnetsClient(
             self.auth_provider, 'network', CONF.identity.region)
+        self.subnetpools_client = subnetpools_client.SubnetpoolsClient(
+            self.auth_provider, 'network', CONF.identity.region)
 
 
 class ZunClient(rest_client.RestClient):
@@ -178,6 +181,19 @@ class ZunClient(rest_client.RestClient):
 
         return url
 
+    @classmethod
+    def networks_uri(cls):
+        url = "/networks/"
+        return url
+
+    @classmethod
+    def network_uri(cls, network_id, params=None):
+        url = "{0}/{1}".format(cls.networks_uri(), network_id)
+        if params:
+            url = cls.add_params(url, params)
+
+        return url
+
     def post_container(self, model, **kwargs):
         """Makes POST /container request
 
@@ -337,6 +353,10 @@ class ZunClient(rest_client.RestClient):
                 return False
         utils.wait_for_condition(is_capsule_in_desired_state, timeout=120)
 
+    def list_capsules(self, **kwargs):
+        resp, body = self.get(self.capsules_uri(), **kwargs)
+        return self.deserialize(resp, body, capsule_model.CapsuleEntity)
+
     def get_capsule(self, capsule_id, params=None):
         resp, body = self.get(self.capsule_uri(capsule_id, params=params))
         return self.deserialize(resp, body, capsule_model.CapsuleEntity)
@@ -344,6 +364,19 @@ class ZunClient(rest_client.RestClient):
     def delete_capsule(self, capsule_id, params=None):
         return self.delete(self.capsule_uri(capsule_id, params=params))
 
+    def ensure_capsule_deleted(self, capsule_id):
+        def is_capsule_deleted():
+            _, model = self.list_capsules()
+            capsule_ids = [c['uuid'] for c in model.capsules]
+            if capsule_id in capsule_ids:
+                return False
+            else:
+                return True
+        utils.wait_for_condition(is_capsule_deleted)
+
+    def delete_network(self, network_id, params=None):
+        return self.delete(self.network_uri(network_id, params=params))
+
 
 @contextlib.contextmanager
 def docker_client(docker_auth_url):
