@@ -19,8 +19,8 @@ from oslo_serialization import jsonutils as json
 from oslo_utils import encodeutils
 import six
 from tempest.lib.common.utils import data_utils
-from tempest.lib.common.utils import test_utils
 from tempest.lib import decorators
+from tempest.lib import exceptions as lib_exc
 
 from zun_tempest_plugin.tests.tempest.api import clients
 from zun_tempest_plugin.tests.tempest.api.common import datagen
@@ -31,7 +31,7 @@ from zun_tempest_plugin.tests.tempest import utils
 class TestContainer(base.BaseZunTest):
 
     credentials = ['primary', 'admin']
-    min_microversion = '1.12'
+    min_microversion = '1.20'
 
     @classmethod
     def get_client_manager(cls, credential_type=None, roles=None,
@@ -46,13 +46,8 @@ class TestContainer(base.BaseZunTest):
     @classmethod
     def setup_clients(cls):
         super(TestContainer, cls).setup_clients()
-        cls.container_client = cls.os_primary.container_client
-        cls.docker_client = clients.DockerClient()
         cls.images_client = cls.os_primary.images_client
-        cls.ports_client = cls.os_primary.ports_client
         cls.sgs_client = cls.os_primary.sgs_client
-        cls.networks_client = cls.os_primary.neutron_client
-        cls.subnets_client = cls.os_primary.subnets_client
         cls.vol_client = cls.os_primary.vol_client
 
     @classmethod
@@ -64,33 +59,18 @@ class TestContainer(base.BaseZunTest):
         self.containers = []
 
     def tearDown(self):
-        hosts = []
         _, model = self.os_admin.container_client.list_containers(
             params={'all_projects': True})
         for c in model.containers:
             if c['uuid'] in self.containers:
-                if c['host'] and c['host'] not in hosts:
-                    hosts.append(c['host'])
                 # NOTE(kiennt): From version 1.7, Zun disallowed non-admin
                 #               users to force delete containers. Therefore,
                 #               we have to be admin to do this action.
                 self.os_admin.container_client.delete_container(
                     c['uuid'],
-                    params={'force': True, 'all_projects': True})
+                    params={'stop': True, 'all_projects': True})
                 self.container_client.ensure_container_deleted(c['uuid'])
 
-        # cleanup the network resources
-        project_id = self.container_client.tenant_id
-        for host in hosts:
-            # NOTE(kiennt): Default docker remote url
-            #               Remove networks in all hosts
-            docker_base_url = self._get_docker_url(host=host)
-            networks = self.docker_client.list_networks(project_id,
-                                                        docker_base_url)
-            for network in networks:
-                self.docker_client.remove_network(network['Id'],
-                                                  docker_base_url)
-
         super(TestContainer, self).tearDown()
 
     @decorators.idempotent_id('b8946b8c-57d5-4fdc-a09a-001d6b552725')
@@ -168,8 +148,8 @@ class TestContainer(base.BaseZunTest):
             model.uuid,
             self._get_docker_url(model))
         env = container.get('Config').get('Env')
-        self.assertTrue('key1=env1', env)
-        self.assertTrue('key2=env2', env)
+        self.assertTrue('key1=env1' in env)
+        self.assertTrue('key2=env2' in env)
 
     @decorators.idempotent_id('0e59d549-58ff-440f-8704-10e223c31cbc')
     def test_run_container_with_labels(self):
@@ -180,8 +160,7 @@ class TestContainer(base.BaseZunTest):
             model.uuid,
             self._get_docker_url(model))
         labels = container.get('Config').get('Labels')
-        self.assertTrue('key1=label1', labels)
-        self.assertTrue('key2=label2', labels)
+        self.assertEqual({'key1': 'label1', 'key2': 'label2'}, labels)
 
     @decorators.idempotent_id('8fc7fec1-e1a2-3f65-a5a6-dba425c1607c')
     def test_run_container_with_port(self):
@@ -189,13 +168,107 @@ class TestContainer(base.BaseZunTest):
         networks = self.networks_client.list_networks()['networks']
         for network in networks:
             if network['project_id'] == project_id:
-                network_id = network['id']
+                tenant_network = network
+                break
+        else:
+            self.fail('Cannot find network in tenant.')
 
-        port = self.ports_client.create_port(network_id=network_id,
-                                             name='testport')
-        _, model = self._run_container(nets=[{'port': 'testport'}])
-        self.assertTrue('testport', model)
-        self.addCleanup(self.ports_client.delete_port, port['port']['id'])
+        port = self.create_port(tenant_network)
+        port_address = port['fixed_ips'][0]['ip_address']
+        port_subnet = port['fixed_ips'][0]['subnet_id']
+
+        _, model = self._run_container(nets=[{'port': port['id']}])
+        address = {'port': port['id'],
+                   'addr': port_address,
+                   'subnet_id': port_subnet}
+        self._assert_container_has_address(model, address,
+                                           network_id=tenant_network['id'])
+
+    @decorators.idempotent_id('cfa24356-30fd-42b7-92c7-bbf01bcaf6eb')
+    def test_run_container_with_port_in_dual_net(self):
+        network = self.create_network()
+        _, subnetv4 = self._create_v4_subnet_and_pool(network)
+        _, subnetv6 = self._create_v6_subnet_and_pool(network)
+        port = self.create_port(network)
+        ipv4_address = None
+        ipv6_address = None
+        for fixed_ip in port['fixed_ips']:
+            if fixed_ip['subnet_id'] == subnetv4['id']:
+                ipv4_address = fixed_ip['ip_address']
+            elif fixed_ip['subnet_id'] == subnetv6['id']:
+                ipv6_address = fixed_ip['ip_address']
+        self.assertIsNotNone(ipv4_address)
+        self.assertIsNotNone(ipv6_address)
+
+        _, model = self._run_container(nets=[{'port': port['id']}])
+
+        self.assertEqual(2, len(model.addresses[network['id']]))
+        address = {'port': port['id'],
+                   'addr': ipv4_address,
+                   'subnet_id': subnetv4['id'],
+                   'version': 4}
+        self._assert_container_has_address(model, address)
+        address = {'port': port['id'],
+                   'addr': ipv6_address,
+                   'subnet_id': subnetv6['id'],
+                   'version': 6}
+        self._assert_container_has_address(model, address)
+
+        # create an ipv4 port from dual net
+        port = self.create_port(network,
+                                fixed_ips=[{'subnet_id': subnetv4['id']}])
+        _, model = self._run_container(nets=[{'port': port['id']}])
+        self.assertEqual(1, len(model.addresses[network['id']]))
+        address = {'port': port['id'],
+                   'addr': port['fixed_ips'][0]['ip_address'],
+                   'subnet_id': subnetv4['id'],
+                   'version': 4}
+        self._assert_container_has_address(model, address)
+
+    def _assert_container_has_address(self, container, address,
+                                      network_id=None):
+        self.assertLessEqual(1, len(container.addresses))
+        if network_id is None:
+            network_id = list(container.addresses.keys())[0]
+        self.assertIn(network_id, container.addresses)
+        for addr in container.addresses[network_id]:
+            if six.viewitems(address) <= six.viewitems(addr):
+                break
+        else:
+            self.fail('Address %s is not found in container %s.' %
+                      (address, container))
+
+    def _create_v4_subnet_and_pool(self, network, subnet_values=None,
+                                   pool_values=None):
+        pool_kwargs = {'prefixes': [u'10.11.12.0/24'],
+                       'min_prefixlen': '28'}
+        if pool_values:
+            pool_kwargs.update(pool_values)
+        created_subnetpool = self.create_subnetpool(**pool_kwargs)
+        pool_id = created_subnetpool['id']
+
+        subnet_kwargs = {'subnetpool_id': pool_id,
+                         'ip_version': 4}
+        if subnet_values:
+            subnet_kwargs.update(subnet_values)
+        subnet = self.create_subnet(network, **subnet_kwargs)
+        return pool_id, subnet
+
+    def _create_v6_subnet_and_pool(self, network, subnet_values=None,
+                                   pool_values=None):
+        pool_kwargs = {'prefixes': [u'2001:db8:3::/48'],
+                       'min_prefixlen': '64'}
+        if pool_values:
+            pool_kwargs.update(pool_values)
+        created_subnetpool = self.create_subnetpool(**pool_kwargs)
+        pool_id = created_subnetpool['id']
+
+        subnet_kwargs = {'subnetpool_id': pool_id,
+                         'ip_version': 6}
+        if subnet_values:
+            subnet_kwargs.update(subnet_values)
+        subnet = self.create_subnet(network, **subnet_kwargs)
+        return pool_id, subnet
 
     @decorators.idempotent_id('9fc7fec0-e1a9-4f65-a5a6-dba425c1607c')
     def test_run_container_with_restart_policy(self):
@@ -207,7 +280,7 @@ class TestContainer(base.BaseZunTest):
             self._get_docker_url(model))
         policy = container.get('HostConfig').get('RestartPolicy')
         self.assertEqual('on-failure', policy['Name'])
-        self.assertTrue(2, policy['MaximumRetryCount'])
+        self.assertEqual(2, policy['MaximumRetryCount'])
 
     @decorators.idempotent_id('58585a4f-cdce-4dbd-9741-4416d1098f94')
     def test_run_container_with_interactive(self):
@@ -222,6 +295,7 @@ class TestContainer(base.BaseZunTest):
         self.assertIs(True, stdin_open)
 
     @decorators.idempotent_id('f181eeda-a9d1-4b2e-9746-d6634ca81e2f')
+    @utils.requires_microversion('1.20')
     def test_run_container_without_security_groups(self):
         gen_model = datagen.container_data()
         delattr(gen_model, 'security_groups')
@@ -251,12 +325,10 @@ class TestContainer(base.BaseZunTest):
         3. Run a container with this network.
         4. Verify container's addresses is in subnet's cidr.
         """
-        test_net = self.networks_client.create_network(
-            name='test_net')['network']
+        test_net = self.create_network(name='test_net')
         self.assertEqual(test_net['name'], 'test_net')
-        test_subnet = self.subnets_client.create_subnet(
-            name='test_subnet', network_id=test_net['id'], ip_version=4,
-            cidr='10.1.0.0/24')['subnet']
+        test_subnet = self.create_subnet(
+            test_net, name='test_subnet', cidr='10.1.0.0/24')
         self.assertEqual(test_subnet['name'], 'test_subnet')
         self.assertEqual(test_subnet['cidr'], '10.1.0.0/24')
         _, model = self._run_container(nets=[{'network': test_net['id']}])
@@ -265,8 +337,6 @@ class TestContainer(base.BaseZunTest):
         addr = list(model.addresses.values())[0][0]['addr']
         self.assertEqual(subnet_id, test_subnet['id'])
         self.assertIn('10.1.0', addr)
-        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
-                        self.networks_client.delete_network, test_net['id'])
 
     @decorators.idempotent_id('2bc86759-ffca-4b3d-bf25-4cf260a67704')
     def test_run_container_with_shared_network(self):
@@ -278,12 +348,13 @@ class TestContainer(base.BaseZunTest):
         3. Run a container with this network.
         4. Verify container's addresses is in subnet's cidr.
         """
-        test_net = self.os_admin.neutron_client.create_network(
-            name='test_net', shared=True)['network']
+        test_net = self.create_network(
+            client=self.os_admin.neutron_client,
+            name='test_net', shared=True)
         self.assertEqual(test_net['name'], 'test_net')
-        test_subnet = self.os_admin.subnets_client.create_subnet(
-            name='test_subnet', network_id=test_net['id'], ip_version=4,
-            cidr='10.1.0.0/24')['subnet']
+        test_subnet = self.create_subnet(
+            test_net, client=self.os_admin.subnets_client,
+            name='test_subnet', cidr='10.1.0.0/24')
         self.assertEqual(test_subnet['name'], 'test_subnet')
         self.assertEqual(test_subnet['cidr'], '10.1.0.0/24')
         _, model = self._run_container(nets=[{'network': test_net['id']}])
@@ -292,9 +363,6 @@ class TestContainer(base.BaseZunTest):
         addr = list(model.addresses.values())[0][0]['addr']
         self.assertEqual(subnet_id, test_subnet['id'])
         self.assertIn('10.1.0', addr)
-        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
-                        self.os_admin.neutron_client.delete_network,
-                        test_net['id'])
 
     @decorators.idempotent_id('7a947d75-ab23-439a-bd94-f6e219f716a9')
     def test_run_container_with_cinder_volumes(self):
@@ -345,6 +413,72 @@ class TestContainer(base.BaseZunTest):
         self.assertEqual(200, resp.status)
         self.assertTrue('hello' in encodeutils.safe_decode(body))
 
+    @decorators.idempotent_id('e49231b2-b095-40d3-9b54-33bb1b371cbe')
+    @utils.requires_microversion('1.20')
+    def test_run_container_with_cinder_volume_dynamic_created(self):
+        """Tests the following:
+
+        1. Run a container with a dynamic-created volume mounted into a path
+           in file system.
+        2. Assert a volume is created in cinder with an attachment to
+           the container.
+        3. Execute a command in the container to write some data to the volume.
+        4. Execute a command in the container to read data from the volume.
+        5. Assert the data read from the volume is the same as the data
+           written in before.
+        6. Delete the container.
+        7. Assert the cinder volume is removed in cinder.
+        """
+        # create a container with the volume
+        container_path = '/data'
+        container_file = '/data/testfile'
+        _, model = self._run_container(mounts=[{
+            'size': 1, 'destination': container_path}])
+        # assert a volume is created in cinder with 'in-use' status.
+        volume_id = None
+        volumes = self.vol_client.list_volumes(detail=True)['volumes']
+        for volume in volumes:
+            for attachment in volume['attachments']:
+                if attachment['server_id'] == model.uuid:
+                    volume_id = volume['id']
+                    break
+        self.assertIsNotNone(volume_id)
+        volume = self.vol_client.show_volume(volume_id)['volume']
+        self.assertEqual('in-use', volume['status'])
+        # write data into the volume
+        resp, _ = self.container_client.exec_container(
+            model.uuid,
+            command="/bin/sh -c 'echo hello > %s'" % container_file)
+        self.assertEqual(200, resp.status)
+        # read data from the volume
+        resp, body = self.container_client.exec_container(
+            model.uuid, command='cat %s' % container_file)
+        self.assertEqual(200, resp.status)
+        self.assertTrue('hello' in encodeutils.safe_decode(body))
+        # delete the container and assert the volume is removed.
+        self.container_client.delete_container(
+            model.uuid, params={'stop': True})
+        self.container_client.ensure_container_deleted(model.uuid)
+        self.assertRaises(lib_exc.NotFound,
+                          self.vol_client.show_volume,
+                          volume_id)
+
+    @decorators.idempotent_id('8a4395ff-3a91-4a35-bd71-5248afc6c465')
+    @utils.requires_microversion('1.25')
+    def test_run_container_with_injected_file(self):
+        # create a container with the volume
+        file_content = 'Random text'
+        container_file = '/data/testfile'
+        _, model = self._run_container(mounts=[{
+            'type': 'bind',
+            'source': utils.encode_file_data(file_content),
+            'destination': container_file}])
+        # read data from the injected file
+        resp, body = self.container_client.exec_container(
+            model.uuid, command='cat %s' % container_file)
+        self.assertEqual(200, resp.status)
+        self.assertTrue(file_content in encodeutils.safe_decode(body))
+
     @decorators.idempotent_id('c3f02fa0-fdfb-49fc-95e2-6e4dc982f9be')
     def test_commit_container(self):
         """Test container snapshot
@@ -357,7 +491,7 @@ class TestContainer(base.BaseZunTest):
         5. Verify the pre-created file is there
         """
         # This command creates a file inside the container
-        command = "/bin/sh -c 'echo hello > testfile;sleep 1000000'"
+        command = ["/bin/sh", "-c", "echo hello > testfile;sleep 1000000"]
         _, model = self._run_container(command=command)
 
         try:
@@ -367,7 +501,7 @@ class TestContainer(base.BaseZunTest):
             self._ensure_image_active('myrepo')
 
             # This command outputs the content of pre-created file
-            command = "/bin/sh -c 'cat testfile;sleep 1000000'"
+            command = ["/bin/sh", "-c", "cat testfile;sleep 1000000"]
             _, model = self._run_container(
                 image="myrepo", image_driver="glance", command=command)
             resp, body = self.container_client.logs_container(model.uuid)
@@ -473,16 +607,16 @@ class TestContainer(base.BaseZunTest):
     @decorators.idempotent_id('a912ca23-14e7-442f-ab15-e05aaa315204')
     def test_logs_container(self):
         _, model = self._run_container(
-            command="/bin/sh -c 'echo hello;sleep 1000000'")
+            command=["/bin/sh", "-c", "echo hello;sleep 1000000"])
         resp, body = self.container_client.logs_container(model.uuid)
         self.assertEqual(200, resp.status)
         self.assertTrue('hello' in encodeutils.safe_decode(body))
 
-    @decorators.skip_because(bug="1764923")
     @decorators.idempotent_id('d383f359-3ebd-40ef-9dc5-d36922790230')
+    @utils.requires_microversion('1.14')
     def test_update_container(self):
         _, model = self._run_container(cpu=0.1, memory=100)
-        self.assertEqual('100M', model.memory)
+        self.assertEqual('100', model.memory)
         self.assertEqual(0.1, model.cpu)
         docker_base_url = self._get_docker_url(model)
         container = self.docker_client.get_container(model.uuid,
@@ -493,21 +627,21 @@ class TestContainer(base.BaseZunTest):
         resp, model = self.container_client.update_container(model.uuid,
                                                              gen_model)
         self.assertEqual(200, resp.status)
-        self.assertEqual('200M', model.memory)
+        self.assertEqual('200', model.memory)
         self.assertEqual(0.2, model.cpu)
         container = self.docker_client.get_container(model.uuid,
                                                      docker_base_url)
         self._assert_resource_constraints(container, cpu=0.2, memory=200)
 
-    @decorators.skip_because(bug="1759459")
     @decorators.idempotent_id('b218bea7-f19b-499f-9819-c7021ffc59f4')
+    @utils.requires_microversion('1.14')
     def test_rename_container(self):
         container1_name = data_utils.rand_name('container1')
         _, model = self._run_container(name=container1_name)
         self.assertEqual(container1_name, model.name)
         container2_name = data_utils.rand_name('container2')
         gen_model = datagen.container_rename_data(name=container2_name)
-        resp, model = self.container_client.rename_container(model.uuid,
+        resp, model = self.container_client.update_container(model.uuid,
                                                              gen_model)
         self.assertEqual(200, resp.status)
         self.assertEqual(container2_name, model.name)
@@ -515,7 +649,7 @@ class TestContainer(base.BaseZunTest):
     @decorators.idempotent_id('142b7716-0b21-41ed-b47d-a42fba75636b')
     def test_top_container(self):
         _, model = self._run_container(
-            command="/bin/sh -c 'sleep 1000000'")
+            command=["/bin/sh", "-c", "sleep 1000000"])
         resp, body = self.container_client.top_container(model.uuid)
         self.assertEqual(200, resp.status)
         self.assertTrue('sleep 1000000' in encodeutils.safe_decode(body))
@@ -533,45 +667,6 @@ class TestContainer(base.BaseZunTest):
         self.assertTrue('MEM %' in encodeutils.safe_decode(body))
         self.assertTrue('BLOCK I/O(B)' in encodeutils.safe_decode(body))
 
-    @decorators.idempotent_id('b3b9cf17-82ad-4c1b-a4af-8210a778a33e')
-    def test_add_and_remove_sg_to_container(self):
-        _, model = self._run_container()
-        sgs = self._get_all_security_groups(model)
-        self.assertEqual(1, len(sgs))
-        self.assertEqual('default', sgs[0])
-        sg_name = data_utils.rand_name('test_add_sg')
-        sg = self.sgs_client.create_security_group(name=sg_name)
-        self.addCleanup(self.sgs_client.delete_security_group,
-                        sg['security_group']['id'])
-        gen_model = datagen.container_add_sg_data(name=sg_name)
-        resp, body = self.container_client.add_security_group(
-            model.uuid, gen_model)
-        self.assertEqual(202, resp.status)
-
-        def assert_security_group_is_added():
-            sgs = self._get_all_security_groups(model)
-            if len(sgs) == 2:
-                self.assertTrue('default' in sgs)
-                self.assertTrue(sg_name in sgs)
-                return True
-            else:
-                return False
-
-        utils.wait_for_condition(assert_security_group_is_added)
-
-        gen_model = datagen.container_remove_sg_data(name=sg_name)
-        resp1, body = self.container_client.remove_security_group(
-            model.uuid, gen_model)
-        self.assertEqual(202, resp1.status)
-
-        def assert_security_group_is_removed():
-            sgs = self._get_all_security_groups(model)
-            if sg_name not in sgs:
-                return True
-            else:
-                return False
-        utils.wait_for_condition(assert_security_group_is_removed)
-
     def _assert_resource_constraints(self, container, cpu=None, memory=None):
         if cpu is not None:
             cpu_quota = container.get('HostConfig').get('CpuQuota')
@@ -681,6 +776,7 @@ class TestContainer(base.BaseZunTest):
         self._ensure_network_attached(model, network)
 
     @decorators.idempotent_id('037d800c-2262-4e15-90cd-95292b5ef958')
+    @utils.requires_microversion('1.1', '1.24')
     def test_put_and_get_archive_from_container(self):
         _, model = self._run_container()
         self.assertEqual(1, len(model.addresses))
@@ -717,6 +813,45 @@ class TestContainer(base.BaseZunTest):
         self.assertEqual(body['stat']['name'], tarinfo.name)
         self.assertEqual(body['stat']['size'], tarinfo.size)
 
+    @decorators.idempotent_id('4ea3a2a5-cf89-48e7-bdd2-0bafc70ca7cb')
+    @utils.requires_microversion('1.25')
+    def test_put_and_get_archive_from_container_encoded(self):
+        _, model = self._run_container()
+        self.assertEqual(1, len(model.addresses))
+
+        # Create a simple tarstream
+        file_content = 'Random text'
+
+        tarstream = BytesIO()
+        with tarfile.open(fileobj=tarstream, mode='w') as tar:
+            encoded_file_content = file_content.encode()
+            tarinfo = tarfile.TarInfo(name='test.txt')
+            tarinfo.size = len(encoded_file_content)
+            tarinfo.mtime = time.time()
+            tar.addfile(tarinfo, BytesIO(encoded_file_content))
+
+        # We're at the end of the tarstream, go back to the beginning
+        tarstream.seek(0)
+
+        req_body = json.dump_as_bytes({
+            'data': utils.encode_file_data(tarstream.getvalue())})
+        resp, _ = self.container_client.put_archive(
+            model.uuid, params={'path': '/tmp'}, body=req_body)
+        self.assertEqual(200, resp.status)
+        resp, body = self.container_client.get_archive(
+            model.uuid, params={'path': '/tmp/test.txt'})
+        self.assertEqual(200, resp.status)
+
+        # Get content
+        body = json.loads(body)
+        tardata = BytesIO(utils.decode_file_data(body['data']))
+        with tarfile.open(fileobj=tardata, mode='r') as tar:
+            untar_content = tar.extractfile('test.txt').read()
+
+        self.assertEqual(file_content, untar_content.decode())
+        self.assertEqual(body['stat']['name'], tarinfo.name)
+        self.assertEqual(body['stat']['size'], tarinfo.size)
+
     def _ensure_network_detached(self, container, network):
         def is_network_detached():
             _, model = self.container_client.get_container(container.uuid)
@@ -736,3 +871,48 @@ class TestContainer(base.BaseZunTest):
                 return False
 
         utils.wait_for_condition(is_network_attached)
+
+
+class TestContainerLegacy(TestContainer):
+
+    credentials = ['primary', 'admin']
+    min_microversion = '1.1'
+    max_microversion = '1.19'
+
+    def _run_container(self, gen_model=None, **kwargs):
+        if 'command' in kwargs and isinstance(kwargs['command'], list):
+            command = ' '.join(["'%s'" % c for c in kwargs['command']])
+            kwargs['command'] = command
+        if gen_model is None:
+            gen_model = datagen.container_data_legacy(**kwargs)
+        resp, model = self.container_client.run_container(gen_model)
+        self.containers.append(model.uuid)
+        self.assertEqual(202, resp.status)
+        # Wait for container to started
+        self.container_client.ensure_container_in_desired_state(
+            model.uuid, 'Running')
+
+        # Assert the container is started
+        resp, model = self.container_client.get_container(model.uuid)
+        self.assertEqual('Running', model.status)
+        self.assertEqual('Running', self._get_container_state(model))
+        return resp, model
+
+    def _create_container(self, **kwargs):
+        if 'command' in kwargs and isinstance(kwargs['command'], list):
+            command = ' '.join(["'%s'" % c for c in kwargs['command']])
+            kwargs['command'] = command
+        gen_model = datagen.container_data_legacy(**kwargs)
+        resp, model = self.container_client.post_container(gen_model)
+        self.containers.append(model.uuid)
+        self.assertEqual(202, resp.status)
+        # Wait for container to finish creation
+        self.container_client.ensure_container_in_desired_state(
+            model.uuid, 'Created')
+
+        # Assert the container is created
+        resp, model = self.container_client.get_container(model.uuid)
+        self.assertEqual(200, resp.status)
+        self.assertEqual('Created', model.status)
+        self.assertEqual('Created', self._get_container_state(model))
+        return resp, model
